/*==============================================================================

FICHIER     : [config.h]

DATE        : 2005/12/0002 21:13:07

CREATEUR    : [Linux!jef]

COMMENTAIRE :

================================================================================

==============================================================================*/

#ifndef __CONFIG_H__
#define __CONFIG_H__

#define KEY_USEMENU	"/apps/dvd95/config/usemenu"
#define KEY_PREVIEW	"/apps/dvd95/config/view"
#define KEY_DOISO	"/apps/dvd95/config/doiso"
#define KEY_DEVICE	"/apps/dvd95/config/dev"
#define KEY_AUDIO	"/apps/dvd95/config/audio"
#define KEY_SUBTITLE	"/apps/dvd95/config/subtitle"
#define KEY_PLAYER	"/apps/dvd95/config/player"
#define KEY_ISODIR	"/apps/dvd95/config/iso"
#define KEY_TEMP	"/apps/dvd95/config/temp"
#define KEY_TRACE	"/apps/dvd95/config/trace"
#define KEY_LANG	"/apps/dvd95/config/lang"
#define KEY_QUALDYN	"/apps/dvd95/config/qualdyn"
#define KEY_GRAVER	"/apps/dvd95/config/graver"
#define KEY_MEDIA	"/apps/dvd95/config/media"
#define KEY_MKISO	"/apps/dvd95/config/mkiso"
#define KEY_GENERIQUE	"/apps/dvd95/config/generique"
#define KEY_SOUND	"/apps/dvd95/config/sound"

#define KEY_MAIN_SX	"/apps/dvd95/config/main-sx"
#define KEY_MAIN_SY	"/apps/dvd95/config/main-sy"
#define KEY_MAIN_PX	"/apps/dvd95/config/main-px"
#define KEY_MAIN_PY	"/apps/dvd95/config/main-py"
#define KEY_MAIN_MAX	"/apps/dvd95/config/main-max"

/*@$#[gconfig.c] global proto. AutoProtoSigV1.1. date: 2010/02/14 00:11:37 */
#include "proto.h"
#ifdef __cplusplus
extern "C" {
#endif
void DialogPropriete(GtkWidget *parent, int mode);
int LoadConfig(GtkWidget *propriete);
int SaveConfig(GtkWidget *propriete);
gchar *ConfigGetString(char *key, char *defValue);
int ConfigGetInt(char *key, int defValue);
int SetConfigString(char *key, char *value);
int SetConfigInt(char *key, int value);
int VerifConfig(GtkWidget *propriete);
void InitTrace(void);
void CreateDirectorys(void);
void ConfigSetSizePosMain(GtkWidget *mainWindow);
void ConfigRecordSizePosMain(GtkWidget *mainWindow);
#ifdef __cplusplus
}
#endif
/*@$% end of AutoProtoSigV1.1 (Dont remove this line) []*/

#endif	/* __CONFIG_H__ */

